:: -------------------------------------------------------------------
:: 
:: install.bat -- Install gnuwin32 package from a local archive
::                whithin the cmd console of windows XP
:: 
:: author : Mathias Michaelis <michaelis@tcnet.ch>
:: date   : May 4, 2006
:: version: 0.6.13
::
:: Copyright (c) 2006 by Mathias Michaelis, <michaelis@tcnet.ch>
::
:: -------------------------------------------------------------------
::
:: Permission is granted to copy, distribute and/or modify this document
:: under the terms of the GNU Free Documentation License, Version 1.2
:: or any later version published by the Free Software Foundation;
:: with no Invariant Sections, no Front-Cover Texts, and no Back-Cover
:: Texts.  A copy of the license is included in the file LICENSE.TXT
::
::
:: Acknowledgments
:: ---------------
::
:: See file README.TXT in this directory. Thanks to all who advised me,
:: gave me some hints, sent me some patches or encouraged me to go on!
::
@echo off
setlocal ENABLEEXTENSIONS
set PATH=%CD%\bin;%CD%\gnuwin32\bin;%PATH%

::
:: Check if there are some packages
::
if not exist packages goto no_packages
dir /a-d /b packages >filelist.tmp 2>nul
if errorlevel 1 goto no_packages

::
:: Make sure that filelist.txt exists. If it does not, create it. 
:: Use the above created directory listing filelist.tmp, sort it
:: and pick only the newest versions.
::
if exist filelist.txt goto prepare

::
:: Pick only relevant files from the list.
::
bin\sed "/^.*-\(bin\|doc\|lib\|dep\)\.zip/!d" filelist.tmp >filelist.txt

::
:: Create the package list from the filelist. To do that
:: 
:: a) Remove the -\(bin\|doc\|lib\|dep\)\.zip at the end of each
::    file name
::    
:: b) Insert tabs between package name, versions and subversion so
::    that the package list can later be sorted by versions
::
bin\sed "s/^\(.*\)-...\.zip$/\1/;s/-[[:digit:]]/\n&/;h;s/^[^\n]*\n*//;s/[[:punct:]]/\t&\t/g;x;s/\n.*$//;G;s/\n//g" filelist.txt >packages.tmp

::
:: Sort the packages, first alphabetically by name, then on each
:: Version with a high priority numerically, then with low priority
:: alphabetically. In this way, the version 7 is older than 7a etc.
::
bin\sort -o packages.txt -k 1,1 -k 3,3n -k 3,3 -k 5,5n -k 5,5 -k 7,7n -k 7,7 -k9,9n -k9,9 -k 11,11n -k 11,11 -k13,13n -k13,13 -k15,15n -k15,15 packages.tmp

::
:: Only pick out the newest versions
::
bin\sed ":x;N;s/^\([^\t\n]*\)\(\t[^\n]*\)\?\n\1\(\t.*\)\?$/\1\3/;tx;P;D" packages.txt >packages.tmp

:: 
:: Remove tabs again.
::
bin\sed "s/\t//g" packages.tmp >packages.txt

::
:: Cancel all packages which name are listed within the include.txt and
:: the exclude.txt files.
::
if exist include.txt bin\sed -n "s/\s*#.*$//;tx;:x;s/^\s*$//;t;s/\s*//g;p" include.txt >include.tmp
if exist include.tmp bin\sed "s/^\(\([^-]*-[^[:digit:]]\)*[^-]*\).*$/\1 -/" include.tmp >exclude.tmp
if exist exclude.txt bin\sed -n "s/\s*#.*$//;tx;:x;s/^\s*$//;t;s/^\s*\([^[:space:]]*\)\s*$/\1 -/p" exclude.txt >>exclude.tmp
if not exist exclude.tmp goto add_include

bin\sort -u packages.txt exclude.tmp -o packages.tmp
bin\sed "/^.* -$/{:x;N;s/^\([^\n]*\) -\n\1-[[:digit:]].*$/\1 -/I;tx;D}" packages.tmp >packages.txt
del packages.tmp
del exclude.tmp

::
:: Add packages in include.txt
::
:add_include
if not exist include.tmp goto pick_files
copy include.tmp+packages.txt packages.tmp >nul
del packages.txt
del include.tmp
ren packages.tmp packages.txt

::
:: Pick out the files from the newest packages
::
:pick_files
bin\sed "s/^.*[^-]$/&-/" packages.txt >packages.tmp
bin\sort -u -o filelist.tmp packages.tmp filelist.txt
bin\sed -n "h;N;s/^\([^\n]\+\)\n\1/\1/;tx;D;:x;G;P;D" filelist.tmp >filelist.txt
del packages.tmp

::
:: Create directories contrib and doc
::
:prepare
del filelist.tmp
if not exist gnuwin32 mkdir gnuwin32
if not exist gnuwin32\contrib mkdir gnuwin32\contrib
if not exist gnuwin32\doc mkdir gnuwin32\doc

::
:: Extract files. Keep contib and doc tidy
::
echo extracting all files ...
cd gnuwin32
..\bin\unzip -u -o -q "..\bin\myaddons.zip"

for /f %%f in (..\filelist.txt) do if exist ..\packages\%%f (
	echo extracting %%f
	..\bin\unzip -u -o -q "..\packages\%%f"
	xcopy /Q /Y /I contrib\*.* "contrib\%%~nf" >nul
	del /F /Q contrib\*.*
	xcopy /Q /Y /I doc\*.* "doc\%%~nf" >nul
	del /F /Q doc\*.*
) else (
	echo %%f doesn't exist!
)
cd ..

::
:: Make a list of info-entries
::
if not exist gnuwin32\bin\install-info.exe goto orphan
if exist gnuwin32\info\dir del gnuwin32\info\dir
echo creating info dir entries ...
dir /a-d /b gnuwin32\info >infolist.tmp 2>nul
if errorlevel 1 goto orphan
bin\sed "/^.*-[[:digit:]]\+\(\.gz\)\?$\|^dir$/d" infolist.tmp >infolist.txt

::
:: Update dir entries of info files
::
for /f %%i in (infolist.txt) do (
	gnuwin32\bin\install-info.exe --infodir=./gnuwin32/info gnuwin32\info\%%i
)

::
:: Remove orphan links, create new ones
::
:orphan
bin\sed "s/^gnuwin32\\bin\\ls -lgGRU gnuwin32 /bin\\ls -lgGRU /;s/\([%]CD\(:[^%]\+\)\?[%]\)\\\\gnuwin32/\1/g;s/gnuwin32\\\{1,2\}//g;" update-links.bat >gnuwin32\update-links.bat

::
:: Here we are!
::
echo .
echo The directory "gnuwin32" can now be copied at any desired placed, e.g. to the
echo directory "%ProgramFiles%\gnuwin32".
echo .
echo Note that copying is the recommended method on an NTFS file system, since the
echo file access rights are not inherited correcty from the parent directory if
echo the files are moved. On a FAT32 file system, there are no access rigths, so
echo nothing speaks against moving.
echo .
echo After moving/copying the gnuwin32 folder at the desired place (may be as
echo administrator) you can run "update-links.bat" from within that (moved/copied)
echo directory. This batch file deletes orphaned links, creates new ones and also
echo creates the folder "gnuwin32\startmenu" that can be placed somewhere in the
echo Windows Start Menu.
echo .
echo Have fun!

goto bye

::
:: Show error massage
::
:no_packages
echo no packages found!

::
:: Tidy-up
::
:bye
if exist filelist.tmp del filelist.tmp
if exist infolist.txt del infolist.txt
if exist infolist.tmp del infolist.tmp

endlocal
